"""
Tests the helper module for updating old simulation variables.
"""
import logging
import os
import shutil
import sqlite3
import unittest

from setup_tests import setUpAll, tearDownAll

from pycoalescence import CoalescenceTree
from pycoalescence.helper import update_parameter_names


class TestHelperUpdate(unittest.TestCase):
    """
    Tests the helper module, with functions for changing simulations between versions
    """

    @classmethod
    def setUpClass(cls):
        """
        Creates the output directory
        """
        setUpAll()

    @classmethod
    def tearDownClass(cls):
        """
        Removes output directory
        """
        tearDownAll()

    def testSimulationUpdateErrors(self):
        """Tests that updating simulation parameters raises the correct errors."""
        with self.assertRaises(IOError):
            update_parameter_names(os.path.join("sample", "PlotBiodiversityMetrics.db"))

    def testDoesntAlterCorrectSim(self):
        """Tests that updating doesn't alter a correct sim."""
        sample_db = os.path.join("sample", "sample.db")
        t = CoalescenceTree(sample_db)
        params1 = t.get_simulation_parameters()
        update_parameter_names(sample_db)
        t = CoalescenceTree(sample_db)
        params2 = t.get_simulation_parameters()
        self.assertEqual(params1, params2)

    def testSimulationUpdating(self):
        """
        Tests that simulation parameters are correctly updated between versions.
        """
        old1 = "sample/data_old1.db"
        old2 = "sample/data_old2.db"
        old3 = "sample/data_old3.db"
        old1a = "output/data_old3.db"
        old2a = "output/data_old4.db"
        old3a = "output/data_old5.db"
        shutil.copy(old1, old1a)
        shutil.copy(old2, old2a)
        shutil.copy(old3, old3a)
        with self.assertRaises(sqlite3.Error):
            t = CoalescenceTree(old1, logging_level=logging.CRITICAL)
            _ = t.get_simulation_parameters()["gen_since_historical"]
        with self.assertRaises(sqlite3.Error):
            t = CoalescenceTree(old2, logging_level=logging.CRITICAL)
            _ = t.get_simulation_parameters()["habitat_change_rate"]
        with self.assertRaises(sqlite3.Error):
            t = CoalescenceTree(old2, logging_level=logging.CRITICAL)
            _ = t.get_simulation_parameters()["habitat_change_rate"]
        update_parameter_names(old1a)
        update_parameter_names(old2a)
        update_parameter_names(old3a)
        t1 = CoalescenceTree(old1a)
        t2 = CoalescenceTree(old2a)
        t3 = CoalescenceTree(old3a)
        self.assertEqual(t1.get_simulation_parameters()["gen_since_historical"], 2.2)
        self.assertEqual(t2.get_simulation_parameters()["gen_since_historical"], 2.2)
        self.assertEqual(t3.get_simulation_parameters()["gen_since_historical"], 2.2)
        self.assertEqual(t1.get_simulation_parameters()["habitat_change_rate"], 0.2)
        self.assertEqual(t2.get_simulation_parameters()["habitat_change_rate"], 0.2)
        self.assertEqual(t3.get_simulation_parameters()["habitat_change_rate"], 0.2)



"""Tests system_operations module for basic system routines."""
import logging
import os
import platform
import sys
import unittest

try:
    if sys.version_info[0] < 3:  # pragma: no cover
        raise ImportError()
    from io import StringIO
except ImportError:
    from io import BytesIO as StringIO

from pycoalescence.system_operations import (
    cantor_pairing,
    check_file_exists,
    check_parent,
    create_logger,
    elegant_pairing,
    execute,
    execute_log_info,
    execute_silent,
    set_logging_method,
)
from setup_tests import setUpAll, tearDownAll, skipLongTest


def setUpModule():
    """
    Creates the output directory and moves logging files
    """
    setUpAll()


def tearDownModule():
    """
    Removes the output directory
    """
    tearDownAll()


@skipLongTest
class TestCantorPairing(unittest.TestCase):
    """
    Tests that cantor pairing successfully creates unique numbers for a very large set of guilds and species
    """

    def testCantorCreatesUniqueIds(self):
        """
        Tests that the cantor pairing function creates unique ids for any given pair of numbers.
        Tests up to 1000 individuals in 1000 guilds
        """
        unique_ids = set()
        for species in range(1000):
            for guild in range(1000):
                ref = cantor_pairing(species, guild)
                self.assertNotIn(ref, unique_ids)
                unique_ids.add(ref)


class TestSystemOperations(unittest.TestCase):
    """
    Tests the functions associated with the system operations.
    """

    def testFileExistence(self):
        """
        Tests that the file existence checks work
        """
        check_file_exists("null")
        check_file_exists("sample/null.tif")
        check_file_exists("test_merger.py")

    def testFileExistenceErrors(self):
        """
        Tests that the correct errors are thrown when files don't exist.
        """
        with self.assertRaises(IOError):
            check_file_exists("not_a_file.tif")
        with self.assertRaises(IOError):
            check_file_exists("sample/not_a_file.tif")
        with self.assertRaises(IOError):
            check_file_exists("sample/also_notta_tile")

    def testCheckDirectory(self):
        """
        Verifies that the parent directory checks work correctly.
        """
        with self.assertRaises(ValueError):
            check_parent("")
        with self.assertRaises(ValueError):
            check_parent(None)
        check_parent("sample/not_here.tif")
        check_parent("output/create_me/create_me_as_well/")
        check_parent("output/create_me_too/file_here.tif")
        self.assertTrue(os.path.exists("output/create_me/"))
        self.assertTrue(os.path.exists("output/create_me/create_me_as_well"))
        self.assertTrue(os.path.exists("output/create_me_too/"))

    def testFileLogger(self):
        """
        Tests that the creation of a logger to file works correctly.
        """
        logger = logging.Logger("temp")
        file_name = "output/log.txt"
        create_logger(logger, file=file_name)
        logger.warning("test output")
        with open(file_name, "r") as f:
            s = f.readline()
            self.assertTrue("test output" in s)
        handlers = logger.handlers[:]
        for handler in handlers:
            handler.close()
            logger.removeHandler(handler)

    def testCantorPairing(self):
        """
        Tests that the cantor pairing produces the expected result
        """
        self.assertEqual(cantor_pairing(10, 100), 6205)
        self.assertEqual(cantor_pairing(0, 1), 2)
        self.assertEqual(cantor_pairing(1, 0), 1)
        self.assertEqual(cantor_pairing(100000, 1000000), 605001550000)

    def testElegantPairing(self):
        """
        Tests the elegant pairing produces the expected output
        """
        self.assertEqual(elegant_pairing(10, 100), 10010)
        self.assertEqual(elegant_pairing(0, 1), 1)
        self.assertEqual(elegant_pairing(1, 0), 2)
        self.assertEqual(elegant_pairing(100000, 1000000), 1000000100000)

    @unittest.skipIf(platform.platform == "Windows", "Skipping tests not compatible with Windows.")
    def testExecute(self):
        """Tests that execution from command line produces the correct output."""
        all_lines = []
        for line in execute(["echo", "123"]):
            all_lines.append(line)
        self.assertEqual(["123\n"], all_lines)

    @unittest.skipIf(platform.platform == "Windows", "Skipping tests not compatible with Windows.")
    def testExecuteSilent(self):
        """Tests that silent execution from command line produces no output."""
        execute_silent(["echo", "123"])
        self.assertTrue(True)

    @unittest.skipIf(platform.platform == "Windows", "Skipping tests not compatible with Windows.")
    def testExecuteInfo(self):
        """Tests that logging info from command line output works as expected."""
        log_stream = StringIO()
        # Remove all handlers associated with the root logger object.
        for handler in logging.root.handlers[:]:
            handler.close()
            logging.root.removeHandler(handler)
        logging.basicConfig(stream=log_stream, level=logging.INFO)
        execute_log_info(["echo", "123"])
        self.assertEqual("INFO:root:123\n", log_stream.getvalue())
        # Remove all handlers associated with the root logger object.
        for handler in logging.root.handlers[:]:
            handler.close()
            logging.root.removeHandler(handler)
        set_logging_method()

    def testLoggingToFile(self):
        """Tests that logging the file works as intended."""
        log_file = os.path.join("output", "log_output1.txt")
        for handler in logging.root.handlers[:]:
            handler.close()
            logging.root.removeHandler(handler)
        set_logging_method(logging_level=logging.INFO, output=log_file)
        logging.info("Test1")
        logging.warning("Test2")
        with open(log_file, "r") as open_log_file:
            self.assertEqual("INFO:Test1\n", open_log_file.readline())
            self.assertEqual("WARNING:Test2\n", open_log_file.readline())
        for handler in logging.root.handlers[:]:
            handler.close()
            logging.root.removeHandler(handler)
        set_logging_method()

    def testLoggingToStream(self):
        """Tests that logging the stream works as intended."""
        log_stream = StringIO()
        # Remove all handlers associated with the root logger object.
        for handler in logging.root.handlers[:]:
            handler.close()
            logging.root.removeHandler(handler)
        logger = create_logger(logging.Logger("temp"), logging_level=logging.INFO, stream=log_stream)
        logger.info("123\n")
        logger.warning("123w\n")
        self.assertEqual("123123w", log_stream.getvalue().replace("\n", ""))
        # Remove all handlers associated with the root logger object.
        for handler in logging.root.handlers[:]:
            handler.close()
            logging.root.removeHandler(handler)
        set_logging_method()



"""Tests the sqlite connection handler object for pycoalescence"""
import os
import sqlite3
import unittest

from setup_tests import setUpAll, tearDownAll

from pycoalescence.sqlite_connection import (
    check_sql_table_exist,
    fetch_table_from_sql,
    SQLiteConnection,
    sql_get_max_from_column,
    check_sql_column_exists,
    get_table_names,
)


def setUpModule():
    """
    Creates the output directory and moves logging files
    """
    setUpAll()


def tearDownModule():
    """
    Removes the output directory
    """
    tearDownAll()


class TestSqlConnection(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """
        Generates the SQLConnection object and tests that it works as intended with the different data types.

        :return:
        """

    def testFilePathOpens(self):
        """
        Tests that the context manager opens correctly with a file path
        """
        with SQLiteConnection("sample/mergers/data_0_0.db") as cursor:
            self.assertTrue(isinstance(cursor, sqlite3.Cursor))
            t = cursor.execute("SELECT max(reference) FROM COMMUNITY_PARAMETERS").fetchone()[0]
            self.assertEqual(6, t)

    def testDatabaseOpens(self):
        """
        Tests that the context manager works with an already-opened database.
        """
        d = sqlite3.connect("sample/mergers/data_0_0.db")
        with SQLiteConnection(d) as cursor:
            self.assertTrue(isinstance(cursor, sqlite3.Cursor))
            t = cursor.execute("SELECT max(reference) FROM COMMUNITY_PARAMETERS").fetchone()[0]
            self.assertEqual(6, t)
        self.assertTrue(isinstance(d, sqlite3.Connection))
        self.assertIsNotNone(d)
        d.close()
        d = None

    def testCursorOpens(self):
        """
        Tests that the context manager works with an already-opened database.
        """
        d = sqlite3.connect("sample/mergers/data_0_0.db")
        c = d.cursor()
        with SQLiteConnection(c) as cursor:
            self.assertTrue(isinstance(cursor, sqlite3.Cursor))
            t = cursor.execute("SELECT max(reference) FROM COMMUNITY_PARAMETERS").fetchone()[0]
            self.assertEqual(6, t)
        self.assertTrue(isinstance(d, sqlite3.Connection))
        self.assertTrue(isinstance(c, sqlite3.Cursor))
        self.assertIsNotNone(d)
        self.assertIsNotNone(c)
        d.close()
        d = None


class TestSqlFetchAndCheck(unittest.TestCase):
    """
    Tests that fetches and checks on the SQL databases work as intended.
    """

    def testReadsCommunityParameters(self):
        """
        Test that community parameters are correctly read from the database.
        """
        community_parameters = fetch_table_from_sql(
            database="sample/mergers/data_0_0.db", table_name="COMMUNITY_PARAMETERS"
        )
        expected_community_parameters = [
            [1, 0.5, 0.0, 0, 0],
            [2, 0.5, 0.5, 0, 0],
            [3, 0.6, 0.0, 0, 0],
            [4, 0.6, 0.5, 0, 0],
            [5, 0.7, 0.0, 0, 0],
            [6, 0.7, 0.5, 0, 0],
        ]
        for i, row in enumerate(expected_community_parameters):
            self.assertListEqual(row, community_parameters[i])
        community_parameters2 = fetch_table_from_sql(
            database="sample/mergers/data_0_0.db", table_name="COMMUNITY_PARAMETERS", column_names=True
        )
        expected_community_parameters2 = [
            ["reference", "speciation_rate", "time", "fragments", "metacommunity_reference"],
            [1, 0.5, 0.0, 0, 0],
            [2, 0.5, 0.5, 0, 0],
            [3, 0.6, 0.0, 0, 0],
            [4, 0.6, 0.5, 0, 0],
            [5, 0.7, 0.0, 0, 0],
            [6, 0.7, 0.5, 0, 0],
        ]
        for i, row in enumerate(expected_community_parameters2):
            self.assertListEqual(row, community_parameters2[i])
        with self.assertRaises(IOError):
            _ = fetch_table_from_sql(database="sample/mergers/data_0_0.db", table_name="COMMUNITY_PARAMETERSXX")

    def testDetectsTablesCorrectly(self):
        """
        Checks that check_sql_table_exist correctly detects the existence of tables
        """
        input_db = os.path.join("sample", "mergers", "data_0_0.db")
        expected_table_names = [
            "COMMUNITY_PARAMETERS",
            "FRAGMENT_OCTAVES",
            "SIMULATION_PARAMETERS",
            "SPECIES_ABUNDANCES",
            "SPECIES_LIST",
            "SPECIES_LOCATIONS",
            "SPECIES_RICHNESS",
        ]
        table_names = get_table_names(input_db)
        table_names.sort()
        self.assertEqual(expected_table_names, table_names)
        self.assertFalse(check_sql_table_exist(database=input_db, table_name="notarealtable"))
        self.assertTrue(check_sql_table_exist(database=input_db, table_name="COMMUNITY_PARAMETERS"))

    def testDetectsColumnCorrectly(self):
        """
        Checks that detection of the column names works as intended.
        """
        self.assertTrue(
            check_sql_column_exists(
                database="sample/mergers/data_0_0.db", table_name="COMMUNITY_PARAMETERS", column_name="reference"
            )
        )
        self.assertFalse(
            check_sql_column_exists(
                database="sample/mergers/data_0_0.db", table_name="COMMUNITY_PARAMETERS", column_name="referencenot"
            )
        )
        self.assertFalse(
            check_sql_column_exists(
                database="sample/mergers/data_0_0.db", table_name="COMMUNITY_PARAMET2ERS", column_name="referencenot"
            )
        )

    def testGetMaxValue(self):
        """
        Tests that the maximum value function works correctly.
        """
        self.assertEqual(6, sql_get_max_from_column("sample/mergers/data_0_0.db", "COMMUNITY_PARAMETERS", "reference"))
        with self.assertRaises(sqlite3.Error):
            self.assertEqual(
                6, sql_get_max_from_column("sample/mergers/data_0_0.db", "COMMUNITY_PARAMETERS", "reference2")
            )
        with self.assertRaises(sqlite3.Error):
            self.assertEqual(
                6, sql_get_max_from_column("sample/mergers/data_0_0.db", "COMMUNITY_PARAMETERS2", "reference")
            )



"""
Simple spatial algorithms required for package functionality.

Algorithms include generation of Voronoi diagrams and spacing points on a landscape using Lloyd's algorithm.
"""
from __future__ import absolute_import

import copy
import logging
import math

try:
    from osgeo import osr, ogr
except ImportError as ie:  # pragma: no cover
    logging.warning("Could not import from osgeo: {}".format(ie))
try:
    from scipy.spatial import Voronoi
except ImportError as ie:  # pragma: no cover
    logging.warning("Cannot import Voronoi from scipy.spatial: {}".format(ie))


def calculate_distance_between(x1, y1, x2, y2):
    """
    Calculates the distance between the points (x1, y1) and (x2, y2)

    .. note:: Returns the absolute value

    :param x1: x coordinate of the first point
    :param y1: y coordinate of the first point
    :param x2: x coordinate of the second point
    :param y2: y coordinate of the second point
    :return: the absolute distance between the points
    """
    return ((x1 - x2) ** 2 + (y1 - y2) ** 2) ** 0.5


def calculate_centre_of_mass(points_list):
    """
    Calculates the centre of mass for the non-intersecting polygon defined by points_list.

    .. note:: the centre of mass will be incorrect for intersecting polygons.

    .. note::

            it is assumed that points_list defines, in order, the vertices of the polygon. The last
            point is assumed to connect to the first point.

    :param points_list: a list of x, y points defining the non-intersecting polygon
    :return: the x,y centre of mass
    """
    centre_x = 0
    centre_y = 0
    area = 0
    size = len(points_list)
    points_list.append(points_list[0])
    for i, each in enumerate(points_list):
        if i == size:
            break
        added_area = each[0] * points_list[i + 1][1] - points_list[i + 1][0] * each[1]
        area += added_area
        centre_x += (each[0] + points_list[i + 1][0]) * added_area
        centre_y += (each[1] + points_list[i + 1][1]) * added_area
    area *= 0.5
    try:
        centre_x /= 6 * area
        centre_y /= 6 * area
    except ZeroDivisionError:
        raise ValueError("Points not ordered in polygon")
    return centre_x, centre_y


def lloyds_algorithm(points_list, maxima, n=7):
    """
    Equally spaces the points in the given landscape defined by (0, x_max), (0, y_max) using Lloyd's algorithm.

    Algorthim is:

    - Reflect the points at x=0, x=x_max, y=0 and y=y_max to make boundaries of the Voronoi diagram on the original
     set of points have finite edges
    - Define the Voronoi diagram separating the points
    - Find the centres of the regions of the voronoi diagram for our original set of points
    - Move the our points to the centres of their voronoi regions
    - Repeat n times (for convergence)
    - Edits the points_list to contain the equally-spaced points

    .. note:: all points are assumed to be in the range x in (0, x_max) and y in (0, y_max)

    :param points_list: a list of points to be equally spaced in the landscape
    :param maxima: the maximum size of the landscape to space out within
    :param n: the number of iterations to perform Lloyd's algorthim for.
    :return list containing the new point centres.
    """
    # Note that lloyd's algorithm is only tested at quite a high level
    for num in range(n):
        vor = Voronoi(reflect_dimensions(points_list, maximums=maxima))
        include_regions = []
        # Find our set of points
        for point_index, point in enumerate(vor.points):
            if 0 <= point[0] < maxima[0] and 0 <= point[1] < maxima[1]:
                include_regions.append(vor.point_region[point_index])
        # Find which regions we want to use
        # Find the centres of our regions
        points_list = []
        for region_index in include_regions:
            vertices_to_average = []
            for vertex in vor.regions[region_index]:
                vertices_to_average.append([vor.vertices[vertex][0], vor.vertices[vertex][1]])
            points_list.append(calculate_centre_of_mass(vertices_to_average))
    return points_list


def reflect_dimensions(points, maximums):
    """
    Reflects the provided points across x=0, y=0, x=x_max and y=y_max (essentially tiling the
    polygon 4 times, around the original polygon).

    :param list points: a list of 2-d points to reflect
    :param tuple maximums: tuple containing the x and y maximums
    :return: a list of reflected points
    """
    max_x, max_y = maximums
    out_points = copy.copy(points)
    for reflection_y in [-0.0001, max_y + 0.0001]:
        for point in points:
            out_points.append([point[0], reflection_y + (reflection_y - point[1])])
    for reflection_x in [-0.0001, max_x + 0.0001]:
        for point in points:
            out_points.append([reflection_x + (reflection_x - point[0]), point[1]])
    return out_points


def archimedes_spiral(centre_x, centre_y, radius, theta):
    """
    Gets the x, y coordinates on a spiral, given a radius and theta

    :param int centre_x: the x coordinate of the centre of the spiral
    :param int centre_y: the y coordinate of the centre of the spiral
    :param float radius: the distance from the centre of the spiral
    :param float theta: the angle of rotation

    :return: tuple of x and y coordinates
    :rtype: tuple
    """
    return int(math.floor(radius * math.cos(theta) + centre_x)), int(math.floor(radius * math.sin(theta) + centre_y))


def convert_coordinates(x, y, input_srs, output_srs):
    """
    Converts the coordinates from the input srs to the output srs.

    :param x: the x coordinate to transform
    :param y: the y coordinate to transform
    :param input_srs: the input srs to transform from
    :param output_srs: the output srs to transform to

    :rtype: list
    :return: transformed [x, y] coordinates
    """
    coord_transform = osr.CoordinateTransformation(input_srs, output_srs)
    # create a geometry from coordinates
    point = ogr.Geometry(ogr.wkbPoint)
    point.AddPoint(x, y)
    point.Transform(coord_transform)
    return point.GetX(), point.GetY()


def estimate_sigma_from_distance(distance, n):
    """
    Estimates the sigma value from a rayleigh distribution (2-d normal) from a total distance travelled in n steps.

    :param float distance: the total distance travelled
    :param int n: the number of steps

    :return: an estimation of the sigma value required to generate the distance travelled in n steps
    """
    return distance * (2.0 / (math.pi * n)) ** 0.5



#  tests for pycoalescence-1.2.9.post2-py38h56dec88_3 (this is a generated file);
print('===== testing package: pycoalescence-1.2.9.post2-py38h56dec88_3 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
"""Runs the basic tests for pycoalescence imports"""
import os
import shutil
import unittest


class TestImports(unittest.TestCase):
    """Tests module imports are successful"""

    def testImportModule(self):
        """Tests that the whole module can be imported"""
        try:
            import pycoalescence
        except ImportError as ie:
            self.fail("Cannot import pycoalescence: {}".format(ie))

    def testImportMain(self):
        """Tests that the main pycoalescence objects can be imported and are not NoneType."""
        try:
            from pycoalescence import Simulation, CoalescenceTree, DispersalSimulation

            for each in ["Simulation", "CoalescenceTree", "DispersalSimulation"]:
                if eval(each) is None:
                    raise ImportError("{} is None.".format(each))
        except ImportError as ie:
            self.fail("Cannot import main pycoalescence objects: {}".format(ie))

    def testImportNecsim(self):
        """Tests that necsim imports correctly."""
        try:
            from pycoalescence.necsim.necsim import CSpatialSimulation

            if CSpatialSimulation is None:
                raise ImportError("CSpatialSimulation is None.")
        except ImportError as ie:
            self.fail("Cannot import c++ libnecsim module: {}".format(ie))


class TestBasicSimulation(unittest.TestCase):
    """Tests that a very basic simulation can be performed. """

    @classmethod
    def setUpClass(cls):
        """Runs a very basic simulation."""
        from pycoalescence import Simulation

        cls.sim = Simulation(logging_level=50)
        cls.sim.set_simulation_parameters(seed=1, task=2, output_directory="tmp", min_speciation_rate=0.9)
        cls.sim.set_map("null", 10, 10)
        cls.sim.run()

    @classmethod
    def tearDownClass(cls):
        """Removes the tmp directory."""
        if os.path.exists("tmp"):
            try:
                shutil.rmtree("tmp")
            except OSError:
                pass

    def testOutputExists(self):
        """Tests that a simulation output database exists at the correct directory."""
        if not os.path.exists(os.path.join("tmp", "data_2_1.db")):
            self.fail("Output database not created.")


if __name__ == "__main__":
    unittest.main()
#  --- run_test.py (end) ---

print('===== pycoalescence-1.2.9.post2-py38h56dec88_3 OK =====');
print("import: 'pycoalescence'")
import pycoalescence

print("import: 'pycoalescence.necsim'")
import pycoalescence.necsim




"""
This module contains the C++ shared object files which are accessed via python.
"""

from pycoalescence.necsim.necsim import CPSpatialSimulation, CPNSESimulation, CSpatialSimulation, CNSESimulation



#!/usr/bin/env python3
#
# This file has been generated by conda-smithy in order to build the recipe
# locally.
#
import os
import glob
import subprocess
from argparse import ArgumentParser


def setup_environment(ns):
    os.environ["CONFIG"] = ns.config
    os.environ["UPLOAD_PACKAGES"] = "False"


def run_docker_build(ns):
    script = ".scripts/run_docker_build.sh"
    subprocess.check_call([script])


def verify_config(ns):
    valid_configs = {os.path.basename(f)[:-5] for f in glob.glob(".ci_support/*.yaml")}
    print(f"valid configs are {valid_configs}")
    if ns.config in valid_configs:
        print("Using " + ns.config + " configuration")
        return
    elif len(valid_configs) == 1:
        ns.config = valid_configs.pop()
        print("Found " + ns.config + " configuration")
    elif ns.config is None:
        print("config not selected, please choose from the following:\n")
        selections = list(enumerate(sorted(valid_configs), 1))
        for i, c in selections:
            print(f"{i}. {c}")
        s = input("\n> ")
        idx = int(s) - 1
        ns.config = selections[idx][1]
        print(f"selected {ns.config}")
    else:
        raise ValueError("config " + ns.config + " is not valid")
    # Remove the following, as implemented
    if not ns.config.startswith("linux"):
        raise ValueError(f"only Linux configs currently supported, got {ns.config}")


def main(args=None):
    p = ArgumentParser("build-locally")
    p.add_argument("config", default=None, nargs="?")

    ns = p.parse_args(args=args)
    verify_config(ns)
    setup_environment(ns)

    run_docker_build(ns)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# pycoalescence documentation build configuration file, created by
# sphinx-quickstart on Mon Oct 24 17:19:28 2016.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import re
import sys

from recommonmark.parser import CommonMarkParser

read_the_docs_build = os.environ.get("READTHEDOCS", None) == "True"

if read_the_docs_build:
    try:
        from unittest.mock import MagicMock
    except ImportError:
        from mock import Mock as MagicMock

    class Mock(MagicMock):
        @classmethod
        def __getattr__(cls, name):
            return MagicMock()

    MOCK_MODULES = ["numpy", "gdal", "pandas", "sqlite3", "osgeo", "libnecsim", "necsimmodule", "matplotlib"]
    sys.modules.update((mod_name, Mock()) for mod_name in MOCK_MODULES)
    sys.modules["scipy"] = Mock()
    sys.modules["scipy.spatial"] = Mock()
    sys.modules["scipy.spatial.Voronoi"] = Mock()
    sys.modules["matplotlib.pyplot"] = Mock()

sys.path.insert(0, os.path.abspath("."))
sys.path.append(os.path.abspath("../"))
sys.path.append(os.path.abspath("../pycoalescence"))

import pycoalescence

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    # 'sphinx.ext.autosectionlabel',
    "sphinx.ext.doctest",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.mathjax",
    "sphinx.ext.ifconfig",
    "sphinx.ext.viewcode",
    "breathe",
    "exhale",
    "nbsphinx",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# Breathe project path
input_dir = "../docs/necsim_doc/xml/"
breathe_projects = {"necsim": input_dir}

# set the default project
breathe_default_project = "necsim"

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = [".rst", ".md"]
# source_suffix = '.rst'
#
source_parsers = {
    ".md": CommonMarkParser,
}

# The encoding of source files.
#
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "pycoalescence"
copyright = pycoalescence.__copyright__
author = pycoalescence.__author__

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = re.sub(r"rc.*", "", pycoalescence.__version__)
# The full version, including alpha/beta/rc tags.
release = pycoalescence.__version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#
# today = ''
#
# Else, today_fmt is used as the format for a strftime call.
#
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    "README_necsim.rst",
    "README.md",
    "*/cmake-*/*"
    # '*class_view_hierarchy.rst',
    "**.ipynb_checkpoints",
]

# The reST default role (used for this markup: `text`) to use for all
# documents.
#
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
# keep_warnings = False

# If true,  produce output, else they produce nothing.
todo_include_todos = True

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
if not read_the_docs_build:
    html_theme = "classic"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.
# "<project> v<release> documentation" by default.
#
# html_title = u'pycoalescence v24/10/16'

# A shorter title for the navigation bar.  Default is the same as html_title.
#
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#
html_logo = "src/PyCoal_logo.png"

# The name of an image file (relative to this directory) to use as a favicon of
# the docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#
html_favicon = "src/PyCoal_favicon_large.ico"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#
# html_extra_path = []

# If not None, a 'Last updated on:' timestamp is inserted at every page
# bottom, using the given strftime format.
# The empty string is equivalent to '%b %d, %Y'.
#
# html_last_updated_fmt = None

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#
# html_additional_pages = {}

# If false, no module index is generated.
#
# html_domain_indices = True

# If false, no index is generated.
#
# html_use_index = True

# If true, the index is split into individual pages for each letter.
#
# html_split_index = False

# If true, links to the reST sources are added to the pages.
#
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Language to be used for generating the HTML full-text search index.
# Sphinx supports the following languages:
#   'da', 'de', 'en', 'es', 'fi', 'fr', 'hu', 'it', 'ja'
#   'nl', 'no', 'pt', 'ro', 'ru', 'sv', 'tr', 'zh'
#
# html_search_language = 'en'

# A dictionary with options for the search language support, empty by default.
# 'ja' uses this config value.
# 'zh' user can custom change `jieba` dictionary path.
#
# html_search_options = {'type': 'default'}

# The name of a javascript file (relative to the configuration directory) that
# implements a search results scorer. If empty, the default will be used.
#
# html_search_scorer = 'scorer.js'

# Output file base name for HTML help builder.
htmlhelp_basename = "pycoalescencedoc"

# -- Options for LaTeX output ---------------------------------------------
preamb_old = r"""
  %\documentclass{memoir}
  \makeatletter
  \fancypagestyle{normal}{
	\fancyhf{}
	\fancyfoot[LE,RO]{{\py@HeaderFamily\thepage}}
	\fancyfoot[LO]{{\py@HeaderFamily\nouppercase{\rightmark}}}
	\fancyfoot[RE]{{\py@HeaderFamily\nouppercase{\leftmark}}}
	\fancyhead[LE,RO]{{\py@HeaderFamily \@title}} % here's the change
	\renewcommand{\headrulewidth}{0.4pt}
	\renewcommand{\footrulewidth}{0.4pt}
  }
  \makeatother
  %\color {blue}
  %\normalcolor {dark blue}
  \pagecolor [RGB]{255, 247, 226}
  \definecolor{VerbatimColor}{rgb}{0.95,0.85,0.65}
  \definecolor{VerbatimBorderColor}{rgb}{0.5,0.95,0.1}
"""
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': preamb_old,
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, "pycoalescence.tex", u"pycoalescence Documentation", u"Samuel Thompson", "manual"),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#
latex_logo = "src/PyCoal_logo.png"

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#
# latex_use_parts = False

# If true, show page references after internal links.
#
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
#
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
#
# latex_appendices = []

# It false, will not define \strong, \code, 	itleref, \crossref ... but only
# \sphinxstrong, ..., \sphinxtitleref, ... To help avoid clash with user added
# packages.
#
# latex_keep_old_macro_names = True

# If false, no module index is generated.
#
# latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "pycoalescence", u"pycoalescence Documentation", [author], 1)]

# If true, show URL addresses after external links.
#
# man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
one_line_description = "Runs and analyses spatially-explicit neutral coalescence simulations"
texinfo_documents = [
    (
        master_doc,
        "pycoalescence",
        u"pycoalescence Documentation",
        author,
        "pycoalescence",
        one_line_description,
        "Miscellaneous",
    ),
]

# Documents to append as an appendix to all manuals.
#
# texinfo_appendices = []

# If false, no module index is generated.
#
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#
# texinfo_no_detailmenu = False


# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = author
epub_copyright = copyright

# The basename for the epub file. It defaults to the project name.
# epub_basename = project

# The HTML theme for the epub output. Since the default themes are not
# optimized for small screen space, using the same theme for HTML and epub
# output is usually not wise. This defaults to 'epub', a theme designed to save
# visual space.
#
# epub_theme = 'epub'

# The language of the text. It defaults to the language option
# or 'en' if the language is not set.
#
# epub_language = ''

# The scheme of the identifier. Typical schemes are ISBN or URL.
# epub_scheme = ''

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A tuple containing the cover image and cover page html template filenames.
#
# epub_cover = ()

# A sequence of (type, uri, title) tuples for the guide element of content.opf.
#
# epub_guide = ()

# HTML files that should be inserted before the pages created by sphinx.
# The format is a list of tuples containing the path and title.
#
# epub_pre_files = []

# HTML files that should be inserted after the pages created by sphinx.
# The format is a list of tuples containing the path and title.
#
# epub_post_files = []

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]

# The depth of the table of contents in toc.ncx.
#
# epub_tocdepth = 3

# Allow duplicate toc entries.
#
# epub_tocdup = True

# Choose between 'default' and 'includehidden'.
#
# epub_tocscope = 'default'

# Fix unsupported image types using the Pillow.
#
# epub_fix_images = False

# Scale large images.
#
# epub_max_image_width = 0

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#
# epub_show_urls = 'inline'

# If false, no index is generated.
#
# epub_use_index = True


# Example configuration for intersphinx: refer to the Python standard library.
# Tell sphinx what the primary language being documented is.
primary_domain = "py"

# Tell sphinx what the pygments highlight language should be.
highlight_language = "py"

# def generateDoxygenXML(stripPath):
# 	'''
# 	Generates the doxygen xml files used by breathe and exhale.
# 	Approach modified from:
#
# 	- https://github.com/fmtlib/fmt/blob/master/doc/build.py
#
# 	:param stripPath:
# 		The value you are sending to exhale.generate via the
# 		key 'doxygenStripFromPath'.  Usually, should be '..'.
# 	'''
# 	from subprocess import PIPE, Popen
# 	if read_the_docs_build:
# 		try:
# 			doxygen_cmd   = ["doxygen", "-"]# "-" tells Doxygen to read configs from stdin
# 			doxygen_proc  = Popen(doxygen_cmd, stdin=PIPE)
# 			doxygen_input = r'''
# 				# Make this the same as what you tell exhale.
# 				OUTPUT_DIRECTORY       = ./Exhaled
# 				# If you need this to be YES, exhale will probably break.
# 				CREATE_SUBDIRS         = NO
# 				# So that only include/ and subdirectories appear.
# 				FULL_PATH_NAMES        = YES
# 				STRIP_FROM_PATH        = "%s/"
# 				# Tell Doxygen where the source code is (yours may be different).
# 				INPUT                  = ../pycoalescence/NECSim/Documentation
# 				# Nested folders will be ignored without this.  You may not need it.
# 				RECURSIVE              = YES
# 				# Set to YES if you are debugging or want to compare.
# 				GENERATE_HTML          = YES
# 				# Unless you want it?
# 				GENERATE_LATEX         = NO
# 				# Both breathe and exhale need the xml.
# 				GENERATE_XML           = YES
# 				# Set to NO if you do not want the Doxygen program listing included.
# 				XML_PROGRAMLISTING     = YES
# 				# Allow for rst directives and advanced functions (e.g. grid tables)
# 				ALIASES                = "rst=\verbatim embed:rst:leading-asterisk"
# 				ALIASES               += "endrst=\endverbatim"
# 			''' % stripPath
# 			# In python 3 strings and bytes are no longer interchangeable
# 			if sys.version[0] == "3":
# 				doxygen_input = bytes(doxygen_input, 'ASCII')
# 			doxygen_proc.communicate(input=doxygen_input)
# 			doxygen_proc.stdin.close()
# 			if doxygen_proc.wait() != 0:
# 				raise RuntimeError("Non-zero return code from 'doxygen'...")
# 		except Exception as e:
# 			raise Exception("Unable to execute 'doxygen': {}".format(e))
# doxy_dir is the parent directory of what you specified in
# `breathe_projects[breathe_default_project]` in `conf.py`

# The configurations you specified
stripPath = ".."
#
# internal_configs = textwrap.dedent('''
#     # Tell doxygen to output wherever breathe is expecting things
#     OUTPUT_DIRECTORY       = {out}
#     # Tell doxygen to strip the path names (RTD builds produce long abs paths...)
#     STRIP_FROM_PATH        = {strip}
# '''.format(out=input_dir, strip=configs.exhaleDoxygenStdin))
# # external_configs = textwrap.dedent(configs.exhaleDoxygenStdin)
# # The full input being sent
# full_input = "{base}\n{internal}\n\n".format(
#     base=configs.DEFAULT_DOXYGEN_STDIN_BASE,
#     # external=external_configs,
#     internal=internal_configs
# )

import codecs

# data="none"
with codecs.open("README_necsim.rst", "r", "utf8") as readme:
    data = readme.read()
# if sys.version_info[0] != 3:
# 	data= data.decode('utf-8', 'ignore')
exhale_args = {
    "containmentFolder": "./necsim",
    "rootFileName": "necsim_library.rst",
    "rootFileTitle": "necsim",
    "doxygenStripFromPath": stripPath,
    "afterTitleDescription": data,
    "createTreeView": True,
    "exhaleExecutesDoxygen": False,
    # "verboseBuild" : True
    # "exhaleDoxygenStdin": '''
    # INPUT=../pycoalescence/lib
    # OUTPUT={}
    # # If you need this to be YES, exhale will probably break.
    # CREATE_SUBDIRS         = NO
    # # So that only Doxygen does not trim paths, which affects the File hierarchy
    # FULL_PATH_NAMES        = YES
    # # Nested folders will be ignored without this.  You may not need it.
    # RECURSIVE              = YES
    # # Set to YES if you are debugging or want to compare.
    # GENERATE_HTML          = YES
    # # Unless you want it...
    # GENERATE_LATEX         = NO
    # # Both breathe and exhale need the xml.
    # GENERATE_XML           = YES
    # # Set to NO if you do not want the Doxygen program listing included.
    # XML_PROGRAMLISTING     = YES
    # # Allow for rst directives and advanced functions e.g. grid tables
    # ALIASES                = "rst=\verbatim embed:rst:leading-asterisk"
    # ALIASES               += "endrst=\endverbatim"
    # # Enable preprocessing and related preprocessor necessities
    # ENABLE_PREPROCESSING   = YES
    # MACRO_EXPANSION        = YES
    # EXPAND_ONLY_PREDEF     = NO
    # SKIP_FUNCTION_MACROS   = NO
    # # extra defs for to help with building the _right_ version of the docs
    # PREDEFINED             = DOXYGEN_DOCUMENTATION_BUILD
    # PREDEFINED            += DOXYGEN_SHOULD_SKIP_THIS
    # # ADDITIONAL
    # EXCLUDE_PATTERNS       = ../pycoalescence/lib/necsim/fast-cpp-csv-parser/* \
    #                     */necsim/fast-cpp-csv-parser/* \
    #                     */lib/autom4te.cache/* \
    #                     */lib/build/* \
    #                     */lib/cmake-*/* \
    #                     */lib/Debug/* \
    #                     */lib/m4/* \
    #                     */lib/Makefiles/* \
    #                     */lib/obj/* \
    #                     */lib/Release/* \
    #                     */lib/SimTest/* \
    #                     *.d \
    #                     *CMakeLists.txt\
    #                     */LICENSE.txt
    # EXCLUDE                = ../pycoalescence/lib/necsim/fast-cpp-csv-parser \
    #                     necsim/fast-cpp-csv-parser/ \
    #                     ../../necsim/fast-cpp-csv-parser/
    # '''
}


# Breathe compatibility
def setup(app):
    pass


# with open("README_NECSim.rst", mode="r") as readme:
# 	data = readme.read()
# stripPath = "../pycoalescence/NECSim/Documentation/"
# generateDoxygenXML(stripPath)
# exhaleArgs = {
# 	"doxygenIndexXMLPath": "../pycoalescence/NECSim/Documentation/xml/index.xml",
# 	"containmentFolder": "./Exhaled",
# 	"rootFileName": "exhaled_library.rst",
# 	"rootFileTitle": "NECSim",
# 	"doxygenStripFromPath": stripPath,
# 	"afterTitleDescription": data,
# 	"createTreeView": True,
# 	"exhaleExecutesDoxygen": True,
# 	"exhaleDoxygenStdin": "INPUT = ../pycoalescence/NECSim"}
# create the dictionary to send to exhale


# import the exhale module from the current directory and generate the necsim
# sys.path.insert(0, os.path.abspath('.'))  # exhale.py is in this directory
# from exhale import generate
# generate(exhaleArgs)
